#include <windows.h> // include for version info constants

A ICON MOVEABLE PURE LOADONCALL DISCARDABLE "../pixmaps/stardict-editor.ico"

//
// TO CHANGE VERSION INFORMATION, EDIT PROJECT OPTIONS...
//
1 VERSIONINFO
FILEVERSION 3,0,7,0
PRODUCTVERSION 3,0,7,0
FILETYPE VFT_APP
{
  BLOCK "StringFileInfo"
	 {
		 BLOCK "040904E4"
		 {
			 VALUE "CompanyName", "http://stardict-4.sourceforge.net"
			 VALUE "FileVersion", "3.0.7"
			 VALUE "FileDescription", "StarDict Editor"
			 VALUE "InternalName", ""
			 VALUE "LegalCopyright", "Hu Zheng <huzheng001@gmail.com>"
			 VALUE "LegalTrademarks", ""
			 VALUE "OriginalFilename", "stardict-editor.exe"
			 VALUE "ProductName", "StarDict Editor"
			 VALUE "ProductVersion", "3.0.7"
		 }
	 }
  BLOCK "VarFileInfo"
	 {
		 VALUE "Translation", 0x0409, 1252
	 }
}
