/*
 * This file was generated by orbit-idl-2 - DO NOT EDIT!
 */

#include <string.h>
#define ORBIT2_STUBS_API
#include "GNOME_Stardict.h"

void GNOME_Stardict_Application_queryWord(GNOME_Stardict_Application _obj, const CORBA_char * str, CORBA_Environment *ev){
gpointer _args[1];
_args[0] = (gpointer)&str;
ORBit_c_stub_invoke (_obj, &GNOME_Stardict_Application__iinterface.methods, 0, NULL, _args, NULL, ev, GNOME_Stardict_Application__classid, G_STRUCT_OFFSET (POA_GNOME_Stardict_Application__epv, queryWord),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Stardict_Application_queryWord);

}
void GNOME_Stardict_Application_grabFocus(GNOME_Stardict_Application _obj, CORBA_Environment *ev){
ORBit_c_stub_invoke (_obj, &GNOME_Stardict_Application__iinterface.methods, 1, NULL, NULL, NULL, ev, GNOME_Stardict_Application__classid, G_STRUCT_OFFSET (POA_GNOME_Stardict_Application__epv, grabFocus),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Stardict_Application_grabFocus);

}
void GNOME_Stardict_Application_hide(GNOME_Stardict_Application _obj, CORBA_Environment *ev){
ORBit_c_stub_invoke (_obj, &GNOME_Stardict_Application__iinterface.methods, 2, NULL, NULL, NULL, ev, GNOME_Stardict_Application__classid, G_STRUCT_OFFSET (POA_GNOME_Stardict_Application__epv, hide),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Stardict_Application_hide);

}
void GNOME_Stardict_Application_quit(GNOME_Stardict_Application _obj, CORBA_Environment *ev){
ORBit_c_stub_invoke (_obj, &GNOME_Stardict_Application__iinterface.methods, 3, NULL, NULL, NULL, ev, GNOME_Stardict_Application__classid, G_STRUCT_OFFSET (POA_GNOME_Stardict_Application__epv, quit),
(ORBitSmallSkeleton) _ORBIT_skel_small_GNOME_Stardict_Application_quit);

}
