/*
 * This file was generated by orbit-idl-2 - DO NOT EDIT!
 */

#ifndef GNOME_Stardict_H
#define GNOME_Stardict_H 1
#include <glib.h>
#define ORBIT_IDL_SERIAL 20
#include <orbit/orbit-types.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/** typedefs **/
#include <bonobo/Bonobo.h>
#if !defined(ORBIT_DECL_GNOME_Stardict_Application) && !defined(_GNOME_Stardict_Application_defined)
#define ORBIT_DECL_GNOME_Stardict_Application 1
#define _GNOME_Stardict_Application_defined 1
#define GNOME_Stardict_Application__freekids CORBA_Object__freekids
typedef CORBA_Object GNOME_Stardict_Application;
extern CORBA_unsigned_long GNOME_Stardict_Application__classid;
#if !defined(TC_IMPL_TC_GNOME_Stardict_Application_0)
#define TC_IMPL_TC_GNOME_Stardict_Application_0 'G'
#define TC_IMPL_TC_GNOME_Stardict_Application_1 'N'
#define TC_IMPL_TC_GNOME_Stardict_Application_2 'O'
#define TC_IMPL_TC_GNOME_Stardict_Application_3 'M'
#define TC_IMPL_TC_GNOME_Stardict_Application_4 'E'
#define TC_IMPL_TC_GNOME_Stardict_Application_5 '_'
#define TC_IMPL_TC_GNOME_Stardict_Application_6 'S'
#define TC_IMPL_TC_GNOME_Stardict_Application_7 't'
#define TC_IMPL_TC_GNOME_Stardict_Application_8 'a'
#define TC_IMPL_TC_GNOME_Stardict_Application_9 'r'
#define TC_IMPL_TC_GNOME_Stardict_Application_10 'd'
#define TC_IMPL_TC_GNOME_Stardict_Application_11 'i'
#define TC_IMPL_TC_GNOME_Stardict_Application_12 'c'
#define TC_IMPL_TC_GNOME_Stardict_Application_13 't'
#ifdef ORBIT_IDL_C_IMODULE_GNOME_Stardict
static
#else
extern
#endif
ORBIT2_MAYBE_CONST struct CORBA_TypeCode_struct TC_GNOME_Stardict_Application_struct;
#define TC_GNOME_Stardict_Application ((CORBA_TypeCode)&TC_GNOME_Stardict_Application_struct)
#endif
#endif

/** POA structures **/
#ifndef _defined_POA_GNOME_Stardict_Application
#define _defined_POA_GNOME_Stardict_Application 1
typedef struct {
  void *_private;
void (*queryWord)(PortableServer_Servant _servant, const CORBA_char * str, CORBA_Environment *ev);
void (*grabFocus)(PortableServer_Servant _servant, CORBA_Environment *ev);
void (*hide)(PortableServer_Servant _servant, CORBA_Environment *ev);
void (*quit)(PortableServer_Servant _servant, CORBA_Environment *ev);
} POA_GNOME_Stardict_Application__epv;
typedef struct {
  PortableServer_ServantBase__epv *_base_epv;
  POA_Bonobo_Unknown__epv *Bonobo_Unknown_epv;
  POA_GNOME_Stardict_Application__epv *GNOME_Stardict_Application_epv;
} POA_GNOME_Stardict_Application__vepv;
typedef struct {
  void *_private;
  POA_GNOME_Stardict_Application__vepv *vepv;
} POA_GNOME_Stardict_Application;
extern void POA_GNOME_Stardict_Application__init(PortableServer_Servant servant, CORBA_Environment *ev);
extern void POA_GNOME_Stardict_Application__fini(PortableServer_Servant servant, CORBA_Environment *ev);
#endif /* _defined_POA_GNOME_Stardict_Application */

/** skel prototypes **/
void _ORBIT_skel_small_GNOME_Stardict_Application_queryWord(POA_GNOME_Stardict_Application *_ORBIT_servant, gpointer _ORBIT_retval, gpointer *_ORBIT_args, CORBA_Context ctx,CORBA_Environment *ev, void (*_impl_queryWord)(PortableServer_Servant _servant, const CORBA_char * str, CORBA_Environment *ev));
void _ORBIT_skel_small_GNOME_Stardict_Application_grabFocus(POA_GNOME_Stardict_Application *_ORBIT_servant, gpointer _ORBIT_retval, gpointer *_ORBIT_args, CORBA_Context ctx,CORBA_Environment *ev, void (*_impl_grabFocus)(PortableServer_Servant _servant, CORBA_Environment *ev));
void _ORBIT_skel_small_GNOME_Stardict_Application_hide(POA_GNOME_Stardict_Application *_ORBIT_servant, gpointer _ORBIT_retval, gpointer *_ORBIT_args, CORBA_Context ctx,CORBA_Environment *ev, void (*_impl_hide)(PortableServer_Servant _servant, CORBA_Environment *ev));
void _ORBIT_skel_small_GNOME_Stardict_Application_quit(POA_GNOME_Stardict_Application *_ORBIT_servant, gpointer _ORBIT_retval, gpointer *_ORBIT_args, CORBA_Context ctx,CORBA_Environment *ev, void (*_impl_quit)(PortableServer_Servant _servant, CORBA_Environment *ev));

/** stub prototypes **/
#define GNOME_Stardict_Application_ref Bonobo_Unknown_ref
#define GNOME_Stardict_Application_unref Bonobo_Unknown_unref
#define GNOME_Stardict_Application_queryInterface Bonobo_Unknown_queryInterface
void GNOME_Stardict_Application_queryWord(GNOME_Stardict_Application _obj, const CORBA_char * str, CORBA_Environment *ev);
void GNOME_Stardict_Application_grabFocus(GNOME_Stardict_Application _obj, CORBA_Environment *ev);
void GNOME_Stardict_Application_hide(GNOME_Stardict_Application _obj, CORBA_Environment *ev);
void GNOME_Stardict_Application_quit(GNOME_Stardict_Application _obj, CORBA_Environment *ev);
#include <orbit/orb-core/orbit-interface.h>

#ifdef ORBIT_IDL_C_IMODULE_GNOME_Stardict
static 
#else
extern 
#endif
ORBit_IInterface GNOME_Stardict_Application__iinterface;
#define GNOME_Stardict_Application_IMETHODS_LEN 4
#ifdef ORBIT_IDL_C_IMODULE_GNOME_Stardict
static 
#else
extern 
#endif
ORBit_IMethod GNOME_Stardict_Application__imethods[GNOME_Stardict_Application_IMETHODS_LEN];

/** IMethods index */

#ifndef __GNOME_Stardict_Application__imethods_index
#define __GNOME_Stardict_Application__imethods_index
typedef enum {
	GNOME_Stardict_Application_queryWord__imethods_index,
	GNOME_Stardict_Application_grabFocus__imethods_index,
	GNOME_Stardict_Application_hide__imethods_index,
	GNOME_Stardict_Application_quit__imethods_index
} GNOME_Stardict_Application__imethods_index;
#endif /* __GNOME_Stardict_Application__imethods_index */

#ifndef __ORBIT_IMETHODS_INDEX
#define __ORBIT_IMETHODS_INDEX
#define ORBIT_IMETHODS_INDEX(m) (m ## __imethods_index)
#endif /* __ORBIT_IMETHODS_INDEX */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#ifndef EXCLUDE_ORBIT_H
#include <orbit/orbit.h>

#endif /* EXCLUDE_ORBIT_H */
#endif
#undef ORBIT_IDL_SERIAL
