/*
 * This file was generated by orbit-idl-2 - DO NOT EDIT!
 */

#include <string.h>
#define ORBIT2_STUBS_API
#define ORBIT_IDL_C_COMMON
#define GNOME_Stardict_COMMON
#include "GNOME_Stardict.h"

static const CORBA_unsigned_long ORBit_zero_int = 0;

#ifndef ORBIT_IDL_C_IMODULE_GNOME_Stardict
void _ORBIT_skel_small_GNOME_Stardict_Application_queryWord(POA_GNOME_Stardict_Application             *_o_servant, gpointer            _o_retval,gpointer           *_o_args,CORBA_Context       _o_ctx,CORBA_Environment  *_o_ev,
void (*_impl_queryWord)(PortableServer_Servant _servant, const CORBA_char * str, CORBA_Environment *ev)) {
_impl_queryWord (_o_servant, *(const CORBA_char * *)_o_args[0], _o_ev);
}
void _ORBIT_skel_small_GNOME_Stardict_Application_grabFocus(POA_GNOME_Stardict_Application             *_o_servant, gpointer            _o_retval,gpointer           *_o_args,CORBA_Context       _o_ctx,CORBA_Environment  *_o_ev,
void (*_impl_grabFocus)(PortableServer_Servant _servant, CORBA_Environment *ev)) {
_impl_grabFocus (_o_servant, _o_ev);
}
void _ORBIT_skel_small_GNOME_Stardict_Application_hide(POA_GNOME_Stardict_Application             *_o_servant, gpointer            _o_retval,gpointer           *_o_args,CORBA_Context       _o_ctx,CORBA_Environment  *_o_ev,
void (*_impl_hide)(PortableServer_Servant _servant, CORBA_Environment *ev)) {
_impl_hide (_o_servant, _o_ev);
}
void _ORBIT_skel_small_GNOME_Stardict_Application_quit(POA_GNOME_Stardict_Application             *_o_servant, gpointer            _o_retval,gpointer           *_o_args,CORBA_Context       _o_ctx,CORBA_Environment  *_o_ev,
void (*_impl_quit)(PortableServer_Servant _servant, CORBA_Environment *ev)) {
_impl_quit (_o_servant, _o_ev);
}

#endif
#if ( (TC_IMPL_TC_GNOME_Stardict_Application_0 == 'G') \
&& (TC_IMPL_TC_GNOME_Stardict_Application_1 == 'N') \
&& (TC_IMPL_TC_GNOME_Stardict_Application_2 == 'O') \
&& (TC_IMPL_TC_GNOME_Stardict_Application_3 == 'M') \
&& (TC_IMPL_TC_GNOME_Stardict_Application_4 == 'E') \
&& (TC_IMPL_TC_GNOME_Stardict_Application_5 == '_') \
&& (TC_IMPL_TC_GNOME_Stardict_Application_6 == 'S') \
&& (TC_IMPL_TC_GNOME_Stardict_Application_7 == 't') \
&& (TC_IMPL_TC_GNOME_Stardict_Application_8 == 'a') \
&& (TC_IMPL_TC_GNOME_Stardict_Application_9 == 'r') \
&& (TC_IMPL_TC_GNOME_Stardict_Application_10 == 'd') \
&& (TC_IMPL_TC_GNOME_Stardict_Application_11 == 'i') \
&& (TC_IMPL_TC_GNOME_Stardict_Application_12 == 'c') \
&& (TC_IMPL_TC_GNOME_Stardict_Application_13 == 't') \
) && !defined(TC_DEF_TC_GNOME_Stardict_Application)
#define TC_DEF_TC_GNOME_Stardict_Application 1
#ifdef ORBIT_IDL_C_IMODULE_GNOME_Stardict
static
#endif
ORBIT2_MAYBE_CONST struct CORBA_TypeCode_struct TC_GNOME_Stardict_Application_struct = {
{&ORBit_TypeCode_epv, ORBIT_REFCOUNT_STATIC},
CORBA_tk_objref,
0,
0,
ORBIT_ALIGNOF_CORBA_POINTER,
0,
0
,
NULL,
CORBA_OBJECT_NIL,
(char *)"Application",
(char *)"IDL:GNOME/Stardict/Application:1.0",
NULL,
NULL,
-1,
0,
0, 0
};
#endif

#ifndef ORBIT_IDL_C_IMODULE_GNOME_Stardict
CORBA_unsigned_long GNOME_Stardict_Application__classid = 0;
#endif

/* Interface type data */

static ORBit_IArg GNOME_Stardict_Application_queryWord__arginfo [] = {
	{ TC_CORBA_string,  ORBit_I_ARG_IN , (char *)"str" }
};

#ifdef ORBIT_IDL_C_IMODULE_GNOME_Stardict
static
#endif
ORBit_IMethod GNOME_Stardict_Application__imethods [] = {
	{
		{ 1, 1, GNOME_Stardict_Application_queryWord__arginfo, FALSE },
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
TC_void, (char *)"queryWord", 9,
		0 | ORBit_I_METHOD_1_WAY
}
, 	{
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
TC_void, (char *)"grabFocus", 9,
		0 | ORBit_I_METHOD_1_WAY
}
, 	{
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
TC_void, (char *)"hide", 4,
		0 | ORBit_I_METHOD_1_WAY
}
, 	{
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
		{ 0, 0, NULL, FALSE },
TC_void, (char *)"quit", 4,
		0 | ORBit_I_METHOD_1_WAY
}
};

static CORBA_string GNOME_Stardict_Application__base_itypes[] = {
(char *)"IDL:Bonobo/Unknown:1.0",
(char *)"IDL:omg.org/CORBA/Object:1.0"
};
#ifdef ORBIT_IDL_C_IMODULE_GNOME_Stardict
static
#endif
ORBit_IInterface GNOME_Stardict_Application__iinterface = {
TC_GNOME_Stardict_Application,{4, 4, GNOME_Stardict_Application__imethods, FALSE},
{2, 2, GNOME_Stardict_Application__base_itypes, FALSE}
};

